@@ -1,9 +1,9 @@
-VERSION = "0.3.11.2"
+VERSION = "0.3.13"
 PYTHONDIR = "/usr/local/lib/python2.5/site-packages"
 BIN_DIR = "/usr/local/bin"
 LIB_DIR = "/usr/local/lib"
 LOCALE_DIR = "${prefix}/share/locale"
 SHARED_DATA_DIR = "/usr/local/share/conduit"
 SHARED_MODULE_DIR = "/usr/local/lib/conduit/modules"
-
+SETTINGS_IMPL = "GConf"
 
